X-Git-Url: https://git.saurik.com/bison.git/blobdiff_plain/affac6132ae6ee354e1497e456b5c81a0c84334f..9b33de72cd2469bedac2b5f0a48a8782571ea9db:/src/symlist.h
diff --git a/src/symlist.h b/src/symlist.h
index ccd6110b..ab9a4b5e 100644
--- a/src/symlist.h
+++ b/src/symlist.h
@@ -1,6 +1,6 @@
/* Lists of symbols for Bison
- Copyright (C) 2002, 2005 Free Software Foundation, Inc.
+ Copyright (C) 2002, 2005, 2006, 2007 Free Software Foundation, Inc.
This file is part of Bison, the GNU Compiler Compiler.
@@ -23,57 +23,98 @@
# define SYMLIST_H_
# include "location.h"
+# include "scan-code.h"
# include "symtab.h"
/* A list of symbols, used during the parsing to store the rules. */
typedef struct symbol_list
{
- /* The symbol. */
- symbol *sym;
+ /**
+ * Whether this node contains a symbol, a semantic type, a \c <*>, or a
+ * \c <>.
+ */
+ enum {
+ SYMLIST_SYMBOL, SYMLIST_TYPE,
+ SYMLIST_DEFAULT_TAGGED, SYMLIST_DEFAULT_TAGLESS
+ } content_type;
+ union {
+ /**
+ * The symbol or \c NULL iff
+ * symbol_list::content_type = SYMLIST_SYMBOL.
+ */
+ symbol *sym;
+ /**
+ * The semantic type iff symbol_list::content_type = SYMLIST_TYPE.
+ */
+ uniqstr type_name;
+ } content;
location location;
- /* The action is attached to the LHS of a rule. */
- const char *action;
- location action_location;
+ /* If this symbol is the generated lhs for a midrule but this is the rule in
+ whose rhs it appears, MIDRULE = a pointer to that midrule. */
+ struct symbol_list *midrule;
- /* Whether this symbol's value is used in the current action. */
- bool used;
+ /* If this symbol is the generated lhs for a midrule and this is that
+ midrule, MIDRULE_PARENT_RULE = a pointer to the rule in whose rhs it
+ appears, and MIDRULE_PARENT_RHS_INDEX = its rhs index (1-origin) in the
+ parent rule. */
+ struct symbol_list *midrule_parent_rule;
+ int midrule_parent_rhs_index;
+
+ /* The action is attached to the LHS of a rule, but action properties for
+ * each RHS are also stored here. */
+ code_props action_props;
/* Precedence/associativity. */
symbol *ruleprec;
int dprec;
int merger;
+ location merger_declaration_location;
/* The list. */
struct symbol_list *next;
} symbol_list;
-/* Create a list containing SYM at LOC. */
-symbol_list *symbol_list_new (symbol *sym, location loc);
+/** Create a list containing \c sym at \c loc. */
+symbol_list *symbol_list_sym_new (symbol *sym, location loc);
+
+/** Create a list containing \c type_name at \c loc. */
+symbol_list *symbol_list_type_new (uniqstr type_name, location loc);
+
+/** Create a list containing a \c <*> at \c loc. */
+symbol_list *symbol_list_default_tagged_new (location loc);
+/** Create a list containing a \c <> at \c loc. */
+symbol_list *symbol_list_default_tagless_new (location loc);
-/* Print it. */
-void symbol_list_print (symbol_list *l, FILE *f);
+/** Print this list.
-/* Prepend SYM at LOC to the LIST. */
-symbol_list *symbol_list_prepend (symbol_list *l,
- symbol *sym,
- location loc);
+ \pre For every node \c n in the list, n->content_type =
+ SYMLIST_SYMBOL. */
+void symbol_list_syms_print (const symbol_list *l, FILE *f);
-/* Free the LIST, but not the symbols it contains. */
-void symbol_list_free (symbol_list *l);
+/** Prepend \c node to \c list. */
+symbol_list *symbol_list_prepend (symbol_list *list, symbol_list *node);
-/* Return its length. */
-unsigned int symbol_list_length (symbol_list *l);
+/** Free \c list, but not the items it contains. */
+void symbol_list_free (symbol_list *list);
-/* Get symbol N in symbol list L. */
+/** Return the length of \c l. */
+int symbol_list_length (symbol_list const *l);
+
+/** Get item \c n in symbol list \c l. */
symbol_list *symbol_list_n_get (symbol_list *l, int n);
/* Get the data type (alternative in the union) of the value for
symbol N in rule RULE. */
uniqstr symbol_list_n_type_name_get (symbol_list *l, location loc, int n);
-/* The symbol N in symbol list L is USED. */
-void symbol_list_n_used_set (symbol_list *l, int n, bool used);
+/** Set the \c \%destructor for \c node as \c code at \c loc. */
+void symbol_list_destructor_set (symbol_list *node, char const *code,
+ location loc);
+
+/** Set the \c \%printer for \c node as \c code at \c loc. */
+void symbol_list_printer_set (symbol_list *node, char const *code,
+ location loc);
#endif /* !SYMLIST_H_ */